@@ -18,7 +18,7 @@
  *
  * The author may be contacted via http://dmalloc.com/
  *
- * $Id: chunk.c,v 1.207 2004/10/19 14:50:43 gray Exp $
+ * $Id: chunk.c,v 1.217 2007/03/25 18:53:41 gray Exp $
  */
 
 /*
@@ -68,11 +68,11 @@
  * Library Copyright and URL information for ident and what programs
  */
 #if IDENT_WORKS
-#ident "@(#) $Copyright: Dmalloc package Copyright 2000 by Gray Watson $"
+#ident "@(#) $Copyright: Dmalloc package Copyright 2007 by Gray Watson $"
 #ident "@(#) $URL: Source for dmalloc available from http://dmalloc.com/ $"
 #else
 static	char	*copyright =
-  "@(#) $Copyright: Dmalloc package Copyright 2000 by Gray Watson $";
+  "@(#) $Copyright: Dmalloc package Copyright 2007 by Gray Watson $";
 static	char	*source_url =
   "@(#) $URL: Source for dmalloc available from http://dmalloc.com/ $";
 #endif
@@ -226,13 +226,16 @@ static	int	random_level(const int max_level)
  *
  * address -> Address we are looking for.
  *
+ * free_b -> Look on the free list otherwise look on the used list.
+ *
  * exact_b -> Set to 1 to find the exact pointer.  If 0 then the
  * address could be inside a block.
  *
  * update_p -> Pointer to the skip_alloc entry we are using to hold
  * the update pointers.
  */
-static	skip_alloc_t	*find_address(const void *address, const int exact_b,
+static	skip_alloc_t	*find_address(const void *address, const int free_b,
+				      const int exact_b,
 				      skip_alloc_t *update_p)
 {
   int		level_c;
@@ -240,7 +243,12 @@ static	skip_alloc_t	*find_address(const void *address, const int exact_b,
   
   /* skip_address_max_level */
   level_c = MAX_SKIP_LEVEL - 1;
-  slot_p = skip_address_list;
+  if (free_b) {
+    slot_p = skip_free_list;
+  }
+  else {
+    slot_p = skip_address_list;
+  }
   
   /* traverse list to smallest entry */
   while (1) {
@@ -365,6 +373,7 @@ static	skip_alloc_t	*find_free_size(const unsigned int size,
   
   /* space should be free */
   if (found_p != NULL && (! BIT_IS_SET(found_p->sa_flags, ALLOC_FLAG_FREE))) {
+    /* sanity check */
     dmalloc_errno = ERROR_ADDRESS_LIST;
     dmalloc_error("find_free_size");
     return NULL;
@@ -407,10 +416,11 @@ static	int	insert_slot(skip_alloc_t *slot_p, const int free_b)
      * the same size which we will be inserting before.
      */
   }
-  else if (find_address(slot_p->sa_mem, 1 /* exact */, update_p) != NULL) {
+  else if (find_address(slot_p->sa_mem, 0 /* used list */, 1 /* exact */,
+			update_p) != NULL) {
     /*
-     * we should not have found it since that means that someone has
-     * the same size and block-num
+     * Sanity check.  We should not have found it since that means
+     * that someone has the same size and block-num.
      */
     dmalloc_errno = ERROR_ADDRESS_LIST;
     dmalloc_error("insert_slot");
@@ -468,7 +478,10 @@ static	void	*alloc_slots(const int level_n)
   /* we need to allocate a new block of the slots of this level */
   block_p = _dmalloc_heap_alloc(BLOCK_SIZE);
   if (block_p == NULL) {
-    /* error code set in _dmalloc_heap_alloc */
+    /*
+     * Sanity check.  Out of heap memory.  Error code set in
+     * _dmalloc_heap_alloc().
+     */
     return NULL;
   }
   memset(block_p, 0, BLOCK_SIZE);
@@ -630,7 +643,7 @@ static	skip_alloc_t	*get_slot(void)
   /* add in all of the unused slots to the linked list */
   admin_mem = alloc_slots(level_n);
   if (admin_mem == NULL) {
-    /* error code set in alloc_slots */
+    /* Sanity check.  Error code set in alloc_slots(). */
     return NULL;
   }
   
@@ -638,8 +651,8 @@ static	skip_alloc_t	*get_slot(void)
   new_p = entry_free_list[level_n];
   if (new_p == NULL) {
     /*
-     * HUH?  This isn't right.  We should have created a whole bunch
-     * of addresses
+     * Sanity check. We should have created a whole bunch of
+     * addresses.
      */
     dmalloc_errno = ERROR_ADDRESS_LIST;
     dmalloc_error("get_slot");
@@ -654,7 +667,7 @@ static	skip_alloc_t	*get_slot(void)
   
   /* now put it in the used list */
   if (! insert_slot(new_p, 0 /* used list */)) {
-    /* error reported above */
+    /* Sanity check.  error code set in insert_slot(). */
     return NULL;
   }
   
@@ -662,8 +675,8 @@ static	skip_alloc_t	*get_slot(void)
   new_p = entry_free_list[level_n];
   if (new_p == NULL) {
     /*
-     * HUH?  This isn't right.  We should have created a whole bunch
-     * of addresses
+     * Sanity check.  We should have created a whole bunch of
+     * addresses.
      */
     dmalloc_errno = ERROR_ADDRESS_LIST;
     dmalloc_error("get_slot");
@@ -722,7 +735,7 @@ static	skip_alloc_t	*insert_address(void *address, const int free_b,
   
   /* now try and insert the slot into the skip-list */
   if (! insert_slot(new_p, free_b)) {
-    /* error code set in insert_slot */
+    /* Sanity check.  error code set in insert_slot(). */
     return NULL;
   }
   
@@ -761,7 +774,6 @@ static	skip_alloc_t	*insert_address(void *address, const int free_b,
 static	int	expand_chars(const void *buf, const int buf_size,
 			     char *out, const int out_size)
 {
-  int			buf_c;
   const unsigned char	*buf_p, *spec_p;
   char	 		*out_p = out, *bounds_p;
   
@@ -769,19 +781,9 @@ static	int	expand_chars(const void *buf, const int buf_size,
   bounds_p = out + out_size;
   
   /* run through the input buffer, counting the characters as we go */
-  for (buf_c = 0, buf_p = (const unsigned char *)buf;; buf_c++, buf_p++) {
-    
-    /* did we reach the end of the buffer? */
-    if (buf_size < 0) {
-      if (*buf_p == '\0') {
-	break;
-      }
-    }
-    else {
-      if (buf_c >= buf_size) {
-	break;
-      }
-    }
+  for (buf_p = (const unsigned char *)buf;
+       buf_p < (const unsigned char *)buf + buf_size;
+       buf_p++) {
     
     /* search for special characters */
     for (spec_p = (unsigned char *)SPECIAL_CHARS + 1;
@@ -1010,8 +1012,6 @@ static	void	log_error_info(const char *now_file,
   pnt_info_t	pnt_info;
   int		out_len, dump_size, offset;
   
-  dmalloc_error(where);
-
   if (slot_p == NULL) {
     prev_file = NULL;
     prev_line = 0;
@@ -1035,6 +1035,7 @@ static	void	log_error_info(const char *now_file,
   if (reason != NULL) {
     dmalloc_message("  error details: %s", reason);
   }
+  
   /* dump the pointer information */
   if (start_user == NULL) {
     dmalloc_message("  from '%s' prev access '%s'",
@@ -1060,6 +1061,8 @@ static	void	log_error_info(const char *now_file,
       || (dmalloc_errno != ERROR_UNDER_FENCE
 	  && dmalloc_errno != ERROR_OVER_FENCE
 	  && dmalloc_errno != ERROR_FREE_OVERWRITTEN)) {
+    /* we call the error function after writing more info to the logfile */
+    dmalloc_error(where);
     return;
   }
   
@@ -1124,7 +1127,8 @@ static	void	log_error_info(const char *now_file,
   /* find the previous pointer in case it ran over */
   if (dmalloc_errno == ERROR_UNDER_FENCE && start_user != NULL) {
     other_p = find_address((char *)start_user - FENCE_BOTTOM_SIZE - 1,
-			   1 /* not exact pointer */, skip_update);
+			   0 /* used list */, 1 /* not exact pointer */,
+			   skip_update);
     if (other_p != NULL) {
       dmalloc_message("  prev pointer '%#lx' (size %u) may have run over from '%s'",
 		      (unsigned long)other_p->sa_mem, other_p->sa_user_size,
@@ -1138,7 +1142,8 @@ static	void	log_error_info(const char *now_file,
 	   && start_user != NULL
 	   && slot_p != NULL) {
     other_p = find_address((char *)slot_p->sa_mem + slot_p->sa_total_size,
-			   1 /* not exact pointer */, skip_update);
+			   0 /* used list */, 1 /* not exact pointer */,
+			   skip_update);
     if (other_p != NULL) {
       dmalloc_message("  next pointer '%#lx' (size %u) may have run under from '%s'",
 		      (unsigned long)other_p->sa_mem, other_p->sa_user_size,
@@ -1147,6 +1152,9 @@ static	void	log_error_info(const char *now_file,
 					      other_p->sa_line));
     }
   }
+  
+  /* we call the error function after writing more info to the logfile */
+  dmalloc_error(where);
 }
 
 /*
@@ -1589,9 +1597,16 @@ static	skip_alloc_t	*get_memory(const unsigned int size)
  *
  * exact_b -> Set to 1 to find the pointer specifically.  Otherwise we
  * can find the pointer inside of an allocation.
+ *
+ * strlen_b -> Make sure that pnt can hold at least a strlen + 1
+ * bytes.  If 0 then ignore.
+ *
+ * min_size -> Make sure that pnt can hold at least that many bytes.
+ * If 0 then ignore.
  */
 static	int	check_used_slot(const skip_alloc_t *slot_p,
-				const void *user_pnt, const int exact_b)
+				const void *user_pnt, const int exact_b,
+				const int strlen_b, const int min_size)
 {
   const char	*file, *name_p, *bounds_p, *mem_p;
   unsigned int	line, num;
@@ -1607,6 +1622,12 @@ static	int	check_used_slot(const skip_alloc_t *slot_p,
   /* get pointer info */
   get_pnt_info(slot_p, &pnt_info);
   
+  /* the user pointer needs to be within the user space */
+  if (user_pnt != NULL && (char *)user_pnt < (char *)pnt_info.pi_user_start) {
+    dmalloc_errno = ERROR_WOULD_OVERWRITE;
+    return 0;
+  }
+  
   /* if we need the exact pointer, make sure that the user_pnt agrees */
   if (exact_b && user_pnt != pnt_info.pi_user_start) {
     dmalloc_errno = ERROR_NOT_START_BLOCK;
@@ -1702,9 +1723,9 @@ static	int	check_used_slot(const skip_alloc_t *slot_p,
   if (file != DMALLOC_DEFAULT_FILE && line != DMALLOC_DEFAULT_LINE) {
     /* NOTE: we don't use strlen here because we might check too far */
     bounds_p = file + MAX_FILE_LENGTH;
-    for (name_p = file; name_p < bounds_p && *name_p != '\0'; name_p++) {
+    for (name_p = file; name_p <= bounds_p && *name_p != '\0'; name_p++) {
     }
-    if (name_p >= bounds_p
+    if (name_p > bounds_p
 	|| name_p < file + MIN_FILE_LENGTH) {
       dmalloc_errno = ERROR_BAD_FILE;
       return 0;
@@ -1725,6 +1746,39 @@ static	int	check_used_slot(const skip_alloc_t *slot_p,
   }
 #endif
   
+  if (strlen_b) {
+    int	equals_okay_b = 0;
+    mem_p = (char *)user_pnt;
+    if (min_size > 0) {
+      bounds_p = mem_p + min_size;
+      /* min_size can be out of bounds as long as we find a \0 beforehand */
+      if (bounds_p > (char *)pnt_info.pi_user_bounds) {
+	bounds_p = (char *)pnt_info.pi_user_bounds;
+      } else {
+	/* we can equals our boundary if our min_size <= user_bounds */
+	equals_okay_b = 1;
+      }
+    } else {
+      bounds_p = (char *)pnt_info.pi_user_bounds;
+    }
+    for (; mem_p < bounds_p; mem_p++) {
+      if (*mem_p == '\0') {
+	break;
+      }
+    }
+    /* mem_p can == bounds_p if we hit the min_size but can't >= user_bounds*/ 
+    if (mem_p > (char *)pnt_info.pi_user_bounds
+	|| ((! equals_okay_b) && mem_p == (char *)pnt_info.pi_user_bounds)) {
+      dmalloc_errno = ERROR_WOULD_OVERWRITE;
+      return 0;
+    }
+  } else if (min_size > 0) {
+    if ((char *)user_pnt + min_size > (char *)pnt_info.pi_user_bounds) {
+      dmalloc_errno = ERROR_WOULD_OVERWRITE;
+      return 0;
+    }
+  }
+  
   return 1;
 }
 
@@ -1947,7 +2001,8 @@ int	_dmalloc_chunk_read_info(const void *user_pnt, const char *where,
   }
   
   /* find the pointer with loose checking for fence */
-  slot_p = find_address(user_pnt, 0 /* not exact pointer */, skip_update);
+  slot_p = find_address(user_pnt, 0 /* used list */, 0 /* not exact pointer */,
+			skip_update);
   if (slot_p == NULL) {
     dmalloc_errno = ERROR_NOT_FOUND;
     log_error_info(NULL, 0, user_pnt, NULL, "finding address in heap", where);
@@ -1955,7 +2010,8 @@ int	_dmalloc_chunk_read_info(const void *user_pnt, const char *where,
   }
   
   /* might as well check the pointer now */
-  if (! check_used_slot(slot_p, user_pnt, 1 /* exact */)) {
+  if (! check_used_slot(slot_p, user_pnt, 1 /* exact */, 0 /* no strlen */,
+			0 /* no min-size */)) {
     /* errno set in check_slot */
     log_error_info(NULL, 0, user_pnt, slot_p, "checking pointer admin", where);
     return 0;
@@ -2063,7 +2119,8 @@ int	_dmalloc_chunk_heap_check(void)
       }
       
       /* now we look up the block and make sure it exists and is valid */
-      slot_p = find_address(block_p, 1 /* exact */, skip_update);
+      slot_p = find_address(block_p, 0 /* used list */, 1 /* exact */,
+			    skip_update);
       if (slot_p == NULL) {
 	dmalloc_errno = ERROR_ADMIN_LIST;
 	dmalloc_error("_dmalloc_chunk_heap_check");
@@ -2127,8 +2184,8 @@ int	_dmalloc_chunk_heap_check(void)
      * now we look up the slot pointer itself and make sure it exists
      * in a valid block
      */
-    block_slot_p = find_address(slot_p, 0 /* not exact pointer */,
-				skip_update);
+    block_slot_p = find_address(slot_p, 0 /* used list */,
+				0 /* not exact pointer */, skip_update);
     if (block_slot_p == NULL) {
       dmalloc_errno = ERROR_ADMIN_LIST;
       dmalloc_error("_dmalloc_chunk_heap_check");
@@ -2155,7 +2212,8 @@ int	_dmalloc_chunk_heap_check(void)
     /* now check the allocation */
     if (checking_list_c == 0) {
       ret = check_used_slot(slot_p, NULL /* no user pnt */,
-			    0 /* loose pnt checking */);
+			    0 /* loose pnt checking */, 0 /* no strlen */,
+			    0 /* no min-size */);
       if (! ret) {
 	/* error set in check_slot */
 	log_error_info(NULL, 0, NULL, slot_p, "checking user pointer",
@@ -2201,15 +2259,17 @@ int	_dmalloc_chunk_heap_check(void)
  * exact_b -> Set to 1 to find the pointer specifically.  Otherwise we
  * can find the pointer inside of an allocation.
  *
+ * strlen_b -> Make sure that pnt can hold at least a strlen + 1
+ * bytes.  If 0 then ignore.
+ *
  * min_size -> Make sure that pnt can hold at least that many bytes.
- * If -1 then do a strlen + 1 for the \0.  If 0 then ignore.
+ * If 0 then ignore.
  */
 int	_dmalloc_chunk_pnt_check(const char *func, const void *user_pnt,
-				 const int exact_b, const int min_size)
+				 const int exact_b, const int strlen_b,
+				 const int min_size)
 {
   skip_alloc_t	*slot_p;
-  unsigned int	min;
-  pnt_info_t	pnt_info;
   
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_TRANS)) {
     if (func == NULL) {
@@ -2222,7 +2282,8 @@ int	_dmalloc_chunk_pnt_check(const char *func, const void *user_pnt,
   }
   
   /* try to find the address */
-  slot_p = find_address(user_pnt, 0 /* not exact pointer */, skip_update);
+  slot_p = find_address(user_pnt, 0 /* used list */, 0 /* not exact pointer */,
+			skip_update);
   if (slot_p == NULL) {
     if (exact_b) {
       dmalloc_errno = ERROR_NOT_FOUND;
@@ -2230,40 +2291,17 @@ int	_dmalloc_chunk_pnt_check(const char *func, const void *user_pnt,
       return 0;
     }
     else {
-      dmalloc_errno = ERROR_NONE;
       return 1;
     }
   }
   
   /* now make sure that the user slot is valid */
-  if (! check_used_slot(slot_p, user_pnt, exact_b)) {
+  if (! check_used_slot(slot_p, user_pnt, exact_b, strlen_b, min_size)) {
     /* dmalloc_error set in check_used_slot */
     log_error_info(NULL, 0, user_pnt, slot_p, "pointer-check", func);
     return 0;
   }
   
-  /* if min_size is < 0 then do a strlen and take into account the \0 */
-  if (min_size != 0) {
-    if (min_size > 0) {
-      min = min_size;
-    }
-    else {
-      /* length of the string + 1 for the \0 */
-      min = strlen((char *)user_pnt) + 1;
-    }
-    
-    /* get info about the pointer */
-    get_pnt_info(slot_p, &pnt_info);
-    
-    /* are we within bounds on the user pointer and size */
-    if ((char *)user_pnt < (char *)pnt_info.pi_user_start
-	|| (char *)user_pnt + min > (char *)pnt_info.pi_user_bounds) {
-      dmalloc_errno = ERROR_WOULD_OVERWRITE;
-      log_error_info(NULL, 0, user_pnt, slot_p, "pointer-check", func);
-      return 0;
-    }
-  }
-  
   return 1;
 }
 
@@ -2530,16 +2568,49 @@ int	_dmalloc_chunk_free(const char *file, const unsigned int line,
   update_p = skip_update;
   
   /* try to find the address with loose match */
-  slot_p = find_address(user_pnt, 0 /* not exact pointer */, skip_update);
+  slot_p = find_address(user_pnt, 0 /* used list */, 0 /* not exact pointer */,
+			skip_update);
   if (slot_p == NULL) {
-    /* not found */
-    dmalloc_errno = ERROR_NOT_FOUND;
+#if FREED_POINTER_DELAY
+    skip_alloc_t	*del_p;
+    
+    /* search the delay list */
+    for (del_p = free_wait_list_head;
+	 del_p != NULL;
+	 del_p = del_p->sa_next_p[0]) {
+      if (del_p->sa_mem <= user_pnt
+	  && (char *)del_p->sa_mem + del_p->sa_total_size > (char *)user_pnt) {
+	pnt_info_t	info;
+	get_pnt_info(del_p, &info);
+	if (info.pi_user_start == user_pnt) {
+	  dmalloc_errno = ERROR_ALREADY_FREE;
+	}
+	else {
+	  dmalloc_errno = ERROR_NOT_FOUND;
+	}
+	break;
+      }
+    }
+    if (del_p == NULL) {
+#endif
+      /* not in the used list so check the free list */
+      if (find_address(user_pnt, 1 /* free list */, 0 /* not exact pointer */,
+		       skip_update) == NULL) {
+	dmalloc_errno = ERROR_NOT_FOUND;
+      }
+      else {
+	dmalloc_errno = ERROR_ALREADY_FREE;
+      }
+#if FREED_POINTER_DELAY
+    }
+#endif
     log_error_info(file, line, user_pnt, NULL, "finding address in heap",
 		   "free");
     return FREE_ERROR;
   }
   
-  if (! check_used_slot(slot_p, user_pnt, 1 /* exact pnt */)) {
+  if (! check_used_slot(slot_p, user_pnt, 1 /* exact pnt */, 0 /* no strlen */,
+			0 /* no min-size */)) {
     /* error set in check slot */
     log_error_info(file, line, user_pnt, slot_p, "checking pointer admin",
 		   "free");
@@ -2550,7 +2621,16 @@ int	_dmalloc_chunk_free(const char *file, const unsigned int line,
     /* error set and dumped in remove_slot */
     return FREE_ERROR;
   }
-  slot_p->sa_flags = ALLOC_FLAG_FREE;
+  if (BIT_IS_SET(slot_p->sa_flags, ALLOC_FLAG_FENCE)) {
+    /*
+     * We need to preserve the fence-post flag because we may need to
+     * properly check for previously freed pointers in the future.
+     */
+    slot_p->sa_flags = ALLOC_FLAG_FREE | ALLOC_FLAG_FENCE;
+  }
+  else {
+    slot_p->sa_flags = ALLOC_FLAG_FREE;
+  }
   
   alloc_cur_pnts--;
   
@@ -2689,7 +2769,8 @@ void	*_dmalloc_chunk_realloc(const char *file, const unsigned int line,
   }
   
   /* find the old pointer with loose checking for fence post stuff */
-  slot_p = find_address(old_user_pnt, 0 /* not exact pointer */, skip_update);
+  slot_p = find_address(old_user_pnt, 0 /* used list */,
+			0 /* not exact pointer */, skip_update);
   if (slot_p == NULL) {
     dmalloc_errno = ERROR_NOT_FOUND;
     log_error_info(file, line, old_user_pnt, NULL, "finding address in heap",
@@ -3132,3 +3213,68 @@ unsigned long	_dmalloc_chunk_count_changed(const unsigned long mark,
   
   return mem_count;
 }
+
+/*
+ * void _dmalloc_chunk_get_stats
+ *
+ * DESCRIPTION:
+ *
+ * Return a number of statistics about the current heap.
+ *
+ * RETURNS:
+ *
+ * None.
+ *
+ * ARGUMENTS:
+ *
+ * heap_low_p <- Pointer to pointer which, if not 0L, will be set to
+ * the low address in the heap.
+ *
+ * heap_high_p <- Pointer to pointer which, if not 0L, will be set to
+ * the high address in the heap.
+ *
+ * total_space_p <- Pointer to an unsigned long which, if not 0L, will
+ * be set to the total space managed by the library including user
+ * space, administrative space, and overhead.
+ *
+ * user_space_p <- Pointer to an unsigned long which, if not 0L, will
+ * be set to the space given to the user process (allocated and free).
+ *
+ * current_allocated_p <- Pointer to an unsigned long which, if not
+ * 0L, will be set to the current allocated space given to the user
+ * process.
+ *
+ * current_pnt_np <- Pointer to an unsigned long which, if not 0L,
+ * will be set to the current number of pointers allocated by the user
+ * process.
+ *
+ * max_allocated_p <- Pointer to an unsigned long which, if not 0L,
+ * will be set to the maximum allocated space given to the user
+ * process.
+ *
+ * max_pnt_np <- Pointer to an unsigned long which, if not 0L, will be
+ * set to the maximum number of pointers allocated by the user
+ * process.
+ *
+ * max_one_p <- Pointer to an unsigned long which, if not 0L, will be
+ * set to the maximum allocated with 1 call by the user process.
+ */
+void	_dmalloc_chunk_get_stats(void **heap_low_p, void **heap_high_p,
+				 unsigned long *total_space_p,
+				 unsigned long *user_space_p,
+				 unsigned long *current_allocated_p,
+				 unsigned long *current_pnt_np,
+				 unsigned long *max_allocated_p,
+				 unsigned long *max_pnt_np,
+				 unsigned long *max_one_p)
+{
+  SET_POINTER(heap_low_p, _dmalloc_heap_low);
+  SET_POINTER(heap_high_p, _dmalloc_heap_high);
+  SET_POINTER(total_space_p, (user_block_c + admin_block_c) * BLOCK_SIZE);
+  SET_POINTER(user_space_p, alloc_current + free_space_bytes);
+  SET_POINTER(current_allocated_p, alloc_current);
+  SET_POINTER(current_pnt_np, alloc_cur_pnts);
+  SET_POINTER(max_allocated_p, alloc_maximum);
+  SET_POINTER(max_pnt_np, alloc_max_pnts);
+  SET_POINTER(max_one_p, alloc_one_max);
+}
