@@ -18,7 +18,7 @@
  *
  * The author may be contacted via http://dmalloc.com/
  *
- * $Id: env.c,v 1.35 2004/01/14 16:17:59 gray Exp $
+ * $Id: env.c,v 1.37 2007/05/14 17:09:20 gray Exp $
  */
 
 /*
@@ -109,7 +109,7 @@ static	long	hex_to_long(const char *str)
  * Break up ADDR_ALL into ADDR_P and ADDR_COUNT_P
  */
 void	_dmalloc_address_break(const char *addr_all, DMALLOC_PNT *addr_p,
-			       long *addr_count_p)
+			       unsigned long *addr_count_p)
 {
   char	*colon_p;
   
@@ -117,7 +117,7 @@ void	_dmalloc_address_break(const char *addr_all, DMALLOC_PNT *addr_p,
   if (addr_count_p != NULL) {
     colon_p = strchr(addr_all, ':');
     if (colon_p != NULL) {
-      *addr_count_p = atoi(colon_p + 1);
+      *addr_count_p = loc_atoul(colon_p + 1);
     }
   }
 }
@@ -146,16 +146,16 @@ void	_dmalloc_start_break(char *start_all, char **start_file_p,
     SET_POINTER(start_file_p, NULL);
     SET_POINTER(start_line_p, 0);
     SET_POINTER(start_iter_p, 0);
-    SET_POINTER(start_size_p, (unsigned long)atol(start_all + 1));
+    SET_POINTER(start_size_p, loc_atoul(start_all + 1));
   }
   else {
     SET_POINTER(start_file_p, NULL);
     SET_POINTER(start_line_p, 0);
     if (start_all[0] == 'c') {
-      SET_POINTER(start_iter_p, (unsigned long)atol(start_all + 1));
+      SET_POINTER(start_iter_p, loc_atoul(start_all + 1));
     }
     else {
-      SET_POINTER(start_iter_p, (unsigned long)atol(start_all));
+      SET_POINTER(start_iter_p, loc_atoul(start_all));
     }
     SET_POINTER(start_size_p, 0);
   }
@@ -166,7 +166,8 @@ void	_dmalloc_start_break(char *start_all, char **start_file_p,
  * string.
  */
 void	_dmalloc_environ_process(const char *env_str, DMALLOC_PNT *addr_p,
-				 long *addr_count_p, unsigned int *debug_p,
+				 unsigned long *addr_count_p,
+				 unsigned int *debug_p,
 				 unsigned long *interval_p, int *lock_on_p,
 				 char **logpath_p, char **start_file_p,
 				 int *start_line_p,
@@ -238,7 +239,7 @@ void	_dmalloc_environ_process(const char *env_str, DMALLOC_PNT *addr_p,
     if (strncmp(this_p, INTERVAL_LABEL, len) == 0
 	&& *(this_p + len) == ASSIGNMENT_CHAR) {
       this_p += len + 1;
-      SET_POINTER(interval_p, (unsigned long)atol(this_p));
+      SET_POINTER(interval_p, loc_atoul(this_p));
       continue;
     }
     
@@ -279,7 +280,7 @@ void	_dmalloc_environ_process(const char *env_str, DMALLOC_PNT *addr_p,
     if (strncmp(this_p, LIMIT_LABEL, len) == 0
 	&& *(this_p + len) == ASSIGNMENT_CHAR) {
       this_p += len + 1;
-      SET_POINTER(limit_p, (unsigned long)atol(this_p));
+      SET_POINTER(limit_p, loc_atoul(this_p));
       continue;
     }
     
@@ -342,7 +343,7 @@ void	_dmalloc_environ_set(char *buf, const int buf_size,
   }
   if (address != NULL) {
     if (addr_count > 0) {
-      buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s%c%#lx:%ld,",
+      buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s%c%#lx:%lu,",
 			    ADDRESS_LABEL, ASSIGNMENT_CHAR, (long)address,
 			    addr_count);
     }
