@@ -18,7 +18,7 @@
  *
  * The author may be contacted via http://dmalloc.com/
  *
- * $Id: return.h,v 1.32 2000/07/21 22:41:22 gray Exp $
+ * $Id: return.h,v 1.37 2005/10/15 14:44:22 gray Exp $
  */
 
 /*
@@ -64,9 +64,9 @@
  * some bogus data -- it seems to return the last return-address or
  * something like that.
  *
- * Alexandre Oliva <oliva at dcc.unicamp.br> recently advised to change
- * the "=g" to a "=m".  If you are having problems, you may want to
- * return to the =g to see if it works.
+ * Alexandre Oliva recently advised to change the "=g" to a "=m".  If
+ * you are having problems, you may want to return to the =g to see if
+ * it works.
  */
 #define GET_RET_ADDR(file)	asm("st %%i7,%0" : \
 				    "=m" (file) : \
@@ -132,8 +132,8 @@
 /******************************* contributions *******************************/
 
 /*
- * For DEC Alphas running OSF.  from Dave Hill <ddhill at zk3.dec.com>
- * and Alexandre Oliva <oliva at dcc.unicamp.br>.  Thanks guys.
+ * For DEC Alphas running OSF.  From Dave Hill and Alexandre Oliva.
+ * Thanks guys.
  */
 #if __alpha
 
@@ -155,7 +155,7 @@
 
 /*
  * For Data General workstations running DG/UX 5.4R3.00 from Joerg
- * Wunsch <joerg_wunsch at julia.tcd-dresden.de>.
+ * Wunsch.
  */
 #ifdef __m88k__
 
@@ -183,7 +183,7 @@
 /*
  * SGI compilers implement a C level method of accessing the return
  * address by simply referencing the __return_address symbol. -- James
- * Bonfield <jkb at mrc-lmb.cam.ac.uk>
+ * Bonfield.
  */
 #if defined(__sgi)
 
@@ -202,7 +202,7 @@
  * "dmalloc.h.3" in such a way that the malloc/calloc/realloc/free
  * definitions would no longer cause the compiler to bark with
  * 'identifier redeclared' (I just put an #ifdef _STDLIB_H ... #endif
- * around those functions).  -- Wim_van_Duuren at stratus.com
+ * around those functions).  -- Wim van Duuren.
  */
 #if defined(_FTX) && defined(i860)
 
@@ -227,8 +227,7 @@ asm void ASM_GET_RET_ADDR(file)
 /*************************************/
 
 /*
- * For HP-UX PA-RISC with gcc/g++ from Jack Perdue
- * <j-perdue at tamu.edu>.
+ * For HP-UX PA-RISC with gcc/g++ from Jack Perdue.
  */
 #if __hpux && __hppa && __GNUC__ > 1
 
@@ -241,7 +240,7 @@ asm void ASM_GET_RET_ADDR(file)
  * just put togther little tidbits of knowledge I found while looking
  * for one to take a stab with this.  Like I said, it seems to work on
  * my g++ code under HPUX11 on a HP V-class system.  YMMV. -- Jack
- * Perdue (j-perdue at tamu.edu)
+ * Perdue.
  */
 #define GET_RET_ADDR(file)  asm("stw %%r2, %0" : "=m" (file) :  );
 
@@ -251,7 +250,6 @@ asm void ASM_GET_RET_ADDR(file)
 
 /*
  * For Powerpc 603 based system running LynxOS 2.3.1 using gcc/gas.
- * From Shawn Carey <smc at questra.com>.
  */
 #if defined(__powerpc__) && defined(__GNUC__) && !defined(__OPTIMIZE__)
 
@@ -271,6 +269,56 @@ do { \
 
 /*************************************/
 
+/*
+ * RH AS2.1 gcc 2.96, tested on a piece of code compiled with icc (Intel
+ * compiler V8) from Didier Remy.
+ */
+#ifdef __ia64__
+#define GET_RET_ADDR(file)  asm("mov %0=b0" : "=g" (file) : /* no inputs */ )
+#endif
+
+/*************************************/
+
+/*
+ * AIX 4.3 RS/6000 from Joe Buehler.
+ */
+#if defined(_AIX)
+static	void	aix_c_get_ret_addr(int i, int *file)
+{
+  int start = *((int *)&start + 4);
+  while (--i >= 0) {
+    start = *((int *)start);
+  }
+  *file = *((int *)start + 2);
+}
+
+#define GET_RET_ADDR(file)	aix_c_get_ret_addr(0, (int *)&file)
+#endif /* _AIX */
+
+/*************************************/
+
+/*
+ * For ARM based machines with gcc/gas 3.3.3 -- Silvester Erdeg.  Used
+ * to be =g which may be for various gcc/gas or arm versions.  Please
+ * let the dmalloc author know if there is a better ifdef combination
+ * for this.
+ */
+#ifdef __arm__
+#define GET_RET_ADDR(file)   asm("str lr, %0" : "=m" (file) : /* no inputs */ )
+#endif
+
+/*************************************/
+
+/*
+ * For Analog Device's Blackfin processors with gcc/gas 3.4.1 --
+ * Silvester Erdeg.
+ */
+#ifdef bfin
+#define GET_RET_ADDR(file)   asm("%0 = RETS;" : "=g" (file) : /* no inputs */ )
+#endif
+
+/*************************************/
+
 #endif /* USE_RETURN_MACROS */
 
 /********************************** default **********************************/
